import 'package:logger4flutter/core/logger_config_impl.dart';
import 'package:logger4flutter/core/logger_impl.dart';
import 'package:logger4flutter/models/abstracts.dart';
import 'package:logger4flutter/models/enums.dart';

typedef Logger LoggerBuilder(String loggerName, LoggerConfig config);

class LoggerFactory {
  static Map<String, Logger> _loggerMap;
  static LoggerBuilder _builder;
  static LoggerConfig _config = LoggerConfigImpl.getInstance();

  static set logBuilder(LoggerBuilder builder) {
    _builder = builder;
  }

  static updateConfig({
    LoggerLevel printLevel = LoggerLevel.off,
    LoggerLevel fileLevel = LoggerLevel.off,
    String fileDirectoryName,
    LoggerLevel serverLevel = LoggerLevel.off,
    String serverUrl,
  }) {
    _config.updateConfig(
      printLevel,
      fileLevel,
      fileDirectoryName,
      serverLevel,
      serverUrl,
    );
  }

  static Logger getLogger(String loggerName) {
    if (_builder == null) {
      _builder = (n, c) => new LoggerImpl(n, c);
    }

    if (_loggerMap == null) {
      _loggerMap = new Map<String, Logger>();
    }

    if (!_loggerMap.containsKey(loggerName)) {
      _loggerMap[loggerName] = _builder(loggerName, _config);
    }

    return _loggerMap[loggerName];
  }
}
