import 'package:ansicolor/ansicolor.dart';
import 'package:logger4flutter/core/log_file.dart';
import 'package:logger4flutter/models/abstracts.dart';
import 'package:logger4flutter/models/enums.dart';
import 'package:logger4flutter/utils/utils.dart';

class LoggerImpl implements Logger {
  factory LoggerImpl(String loggerName, LoggerConfig loggerConfig) {
    var logFile =
        LogFile.getInstance(directoryName: loggerConfig.fileDirectoryName);
    return new LoggerImpl._(loggerName, loggerConfig, logFile);
  }

  LoggerImpl._(this.name, this._config, this._logFile);

  final String name;
  final LoggerConfig _config;
  final LogFile _logFile;

  @override
  trace(String message) {
    LoggerLevel level = LoggerLevel.trace;
    _log(message, level);
  }

  @override
  debug(String message) {
    LoggerLevel level = LoggerLevel.debug;
    _log(message, level);
  }

  @override
  info(String message) {
    LoggerLevel level = LoggerLevel.info;
    _log(message, level);
  }

  @override
  warn(String message) {
    LoggerLevel level = LoggerLevel.warn;
    _log(message, level);
  }

  @override
  error(String message) {
    LoggerLevel level = LoggerLevel.error;
    _log(message, level);
  }

  @override
  fatal(String message) {
    LoggerLevel level = LoggerLevel.fatal;
    _log(message, level);
  }

  _log(String message, LoggerLevel level) {
    if (_config.printLevel.index < level.index) {
      AnsiPen ansiPen = LoggerUtils.getColor(level);
      ansiPen(message);
    }
    if (_config.fileLevel.index < level.index) {
      _logFile.writeLog(message);
    }
  }
}
